#!/usr/bin/python

from optparse import OptionParser
from util import gtkcrashhandler

gtkcrashhandler.initialize(app_name="Strain Analyser")

parser = OptionParser()
parser.add_option("-l", "--livecoding",
		dest="livecoding", action="store_true", default=False,
		help="use livecoding module to aid code editing and debugging")

(options, args) = parser.parse_args()

if options.livecoding:
	import logging
	logging.basicConfig(level=logging.DEBUG)

	logger = logging.getLogger("strainanalyser")
	logger.warning("loading livecoding in strainanalyser/")

	from livecoding import reloader

	import sys, os
	# don't want to accidentally import strainanalyser directly, rather than through livecoding!
	sys.path.remove(os.getcwd())
	logger.debug("sys.path = %s" % sys.path)

	cr = reloader.CodeReloader()


	# livecoding defaults to putting all files' contents in the same module,
	# rather than Python's default of one-module-per-file; override this to
	# match Python's default to avoid having two sets of import statements

	class MyReloadableScriptFile(reloader.ReloadableScriptFile):

		def __init__(self, filePath, namespacePath, implicitLoad=True):
			# ignore whatever namespace livecoding has come up with; figure it out ourselves
			# (and assume that namespace = path...)
			myNamespace = os.path.splitext(filePath)[0].replace(os.sep, ".")
			super(MyReloadableScriptFile, self).__init__(filePath, myNamespace, implicitLoad)

		def Run(self):
			# also it misses out magic globals...
			self.scriptGlobals = {"__file__": self.filePath}
			# the following code copied from namespace.ScriptFile.Run...
			try:
				eval(self.codeObject, self.scriptGlobals, self.scriptGlobals)
			except (ImportError, AttributeError):
				import sys, traceback
				self.lastError = traceback.format_exception(*sys.exc_info())
				return False
			return True


	class MyReloadableScriptDirectory(reloader.ReloadableScriptDirectory):

		scriptFileClass = MyReloadableScriptFile

		def SetModuleAttributes(self, scriptFile, namespace, overwritableAttributes=set()):
			# the code already sets __file__ but too late, and then complains about the one I added above!
			# => allow my one to override the default one
			overwritableAttributes.add("__file__")
			super(MyReloadableScriptDirectory, self).SetModuleAttributes(scriptFile, namespace, overwritableAttributes)


	cr.scriptDirectoryClass = MyReloadableScriptDirectory

	cr.AddDirectory("strainanalyser", "strainanalyser")

from strainanalyser.datasetdiff import run
run()

