#!/usr/bin/python

import gtk

from strainanalyser.util import *
from strainanalyser.dataset import *
from strainanalyser.datahandling import *
from strainanalyser.graphing import *


class StrainAnalyserDatasetDiffUI(AbstractUI):

	def __init__(self):
		super(StrainAnalyserDatasetDiffUI, self).__init__("datasetdiff.xml", "datasetdiffwindow")

		self.datasetcollection = StrainAnalyserDatasetCollection()
		self.diff = StrainAnalyserDiffDatasetCollection(self.datasetcollection)

		self.datagraph = StrainAnalyserDatasetDiffGraph(
				container_vbox=self.get_object("graphcontainer_vbox"),
				dataset_diff_collection = self.diff
			)

		# initialise dataset list model-view-controller stuff
		widget = self.get_object("datasetlist")
		cols = [
				("Name", 0, True),
				("Resolution (m)", 3, False),
			]
		for (title, columnId, expand) in cols:
			column = gtk.TreeViewColumn(title, gtk.CellRendererText(), text=columnId)
			column.set_expand(expand)
			widget.append_column(column)

		radio_renderer = gtk.CellRendererToggle()
		radio_renderer.set_radio(True)
		radio_renderer.set_property('activatable', True)
		radio_renderer.connect("toggled", self.on_datasetlist_ref_toggled)
		column = gtk.TreeViewColumn("Reference", radio_renderer, active=1)
		widget.append_column(column)

		widget.set_model(self.datasetcollection)

		self.get_object("noisefilt").set_active(self.diff.noisefilt_enabled)
		self.get_object("noisefilt_order").set_value(self.diff.noisefilt_order)
		self.get_object("noisefilt_kernel").set_value(self.diff.noisefilt_kernel)

	# Signal handlers

	def on_datasetdiffwindow_destroy(self, widget):
		gtk.main_quit()

	def on_open_dataset_clicked(self, widget):
		# TODO
		self.datagraph.redraw()

	def on_new_dataset_clicked(self, widget):
		def cb(cb_arg, dataset):
			self.datasetcollection.add(dataset)
			self.datagraph.redraw()

		StrainAnalyserDatasetUI(save_cb=cb).show()

	def on_edit_dataset_clicked(self, widget):
		datasetlist = self.get_object("datasetlist")
		cursor = datasetlist.get_cursor()[0]
		if cursor:
			index = cursor[0]
			row = self.datasetcollection[index]
			dataset = row[2]

			def cb(cb_arg, dataset):
				row[0] = dataset.name
				row[3] = dataset.resolution
				self.datagraph.redraw()

			StrainAnalyserDatasetUI(compensated_dataset=dataset, save_cb=cb).show()

	def on_remove_dataset_clicked(self, widget):
		datasetlist = self.get_object("datasetlist")
		cursor = datasetlist.get_cursor()[0]
		if cursor:
			index = cursor[0]
			self.datasetcollection.remove(index)
			self.datagraph.redraw()

	def on_up_dataset_clicked(self, widget):
		datasetlist = self.get_object("datasetlist")
		cursor = datasetlist.get_cursor()[0]
		if cursor:
			index = cursor[0]
			self.datasetcollection.move_up(index)
			self.datagraph.redraw()

	def on_down_dataset_clicked(self, widget):
		datasetlist = self.get_object("datasetlist")
		cursor = datasetlist.get_cursor()[0]
		if cursor:
			index = cursor[0]
			self.datasetcollection.move_down(index)
			self.datagraph.redraw()

	def on_noisefilt_toggled(self, widget):
		self.diff.noisefilt_enabled = widget.get_active()
		for i in ["order", "kernel"]:
			for j in ["slider", "label"]:
				self.get_object("noisefilt_%s_%s" % (i,j)).set_sensitive(widget.get_active())
		self.datagraph.redraw()

	def on_noisefilt_order_slider_value_changed(self, widget):
		self.diff.noisefilt_order = widget.get_value()
		self.datagraph.redraw()

	def on_noisefilt_kernel_slider_value_changed(self, widget):
		self.diff.noisefilt_kernel = widget.get_value()
		self.datagraph.redraw()

	def on_datasetlist_ref_toggled(self, widget, path):
		# update dataset collection (have to do the radio button logic ourselves...)
		if self.datasetcollection[path][1]:
			# clicked the row that's already reference -- we don't want a ref any more (hybrid radio button)
			self.datasetcollection[path][1] = False
		else:
			# clicked a row that should become reference -- turn everything off and this row on
			for row in self.datasetcollection:
				row[1] = False
			self.datasetcollection[path][1] = True
		self.datagraph.redraw()


def run():
	ui = StrainAnalyserDatasetDiffUI()
	ui.show()
	gtk.main()

if __name__ == "__main__":
	run()

